package rabinizer.exec;
import java.util.*;

import rabinizer.formulas.*;

public class KRabinStatesAcceptanceCondition {
	
	
	 public List<Tuple<Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>, Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>>> condition = null;      
	
		
	KRabinStatesAcceptanceCondition(KRabinAutomatonStates rabin){
		condition = new ArrayList<Tuple<Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>, Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>>>(rabin.rCondition.size());
		
		for(int i=0; i<rabin.rCondition.size();i++){
			Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>> empty = new HashSet<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>();
			Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>> empty2 = new HashSet<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>();
			Tuple<Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>, Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>> emptyTuple = new Tuple<Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>, Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>>(empty,empty2);
			condition.add(i,emptyTuple);
					
		}
		
		for (Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>> state : rabin.states) {
			for(int i=0; i<rabin.rCondition.size();i++){
				if(state.right.get(i)==1){
					condition.get(i).left.add(state);
				}
				if(state.right.get(rabin.rCondition.size()+i)==1){
					condition.get(i).right.add(state);
				}
				
			}
			
		}
	
	}
		
		
	
	public String toString() {
        int rpc = 0;
        String r = "Rabin pair count: "+condition.size()+"\n";
        for (Tuple<Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>, Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>> pair : condition) {
            r += "Pair "+(++rpc)+":\n";
        	r += pairToString(pair);
        }
        return r;
    }
    
	
	
    private String pairToString(Tuple<Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>, Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>> pair) {
    	String r = "[\n";
        for (Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>> avoidRabinState : pair.left)
            r += " "+avoidRabinState+"\n";
        r += "]\n[\n";
        for (Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>> reachRabinState : pair.right) {
        	//r += " [\n";
        	//for (Tuple<Formula, KState> reachGenRabinState : reachRabinState.getKey())
        		r += "  "+reachRabinState+"\n";
        	//r += " ]" + reachRabinState.getValue()+"\n";
        }
        r += "]\n";
        return r;
    }
    
    public int size() {
    	return condition.size();
    }
    
    private String stateSetToNumbers (Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>> stateSet, Map<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>, Integer> ordering) {
    	String result = "{";
    	boolean first = true;
    	for (Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>> state : stateSet)
    		if(first){
    			result += ordering.get(state);
    			first = false;
    		} else
    			result += ", " + ordering.get(state);
    	return result + "}";
    }
    
    public String toPrism (Map<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>, Integer> ordering) {
    	String result = "";
    	for (Tuple<Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>, Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>> pair : condition)         
            	result += " (" + stateSetToNumbers(pair.left, ordering) + "," + stateSetToNumbers(pair.right, ordering) +")\n";
    	return result;
    }
    
    
    public String toNumericalString() {
        String r = "";
        Integer rpc2 = 0;
        for (Tuple<Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>, Set<Tuple<Tuple<Tuple<Formula, KState>, List<Integer>>,List<Integer>>>> pair : condition) {
            if (rpc2 > 0) {
                r += " | ";
            }
            r += "(Fin(" + rpc2 + ")&Inf(" + (++rpc2) + "))";
            rpc2++;
        }
        return (rpc2) + " " + r;
    }
  
}
