package rabinizer.formulas;

import java.util.*;

import net.sf.javabdd.BDD;
import rabinizer.exec.*;

public class XOperator extends Formula {

	public Formula operand;

	public XOperator(Formula f) {
		this.operand = f;
	}
	
	public BDD bdd() {
		if (cachedBdd == null) {
			Formula booleanAtom = new XOperator(operand.representative());
			int bddVar = Misc.bijectionBooleanAtomBddVar.id(booleanAtom);
			if (Misc.bddFactory.varNum() <= bddVar)	Misc.bddFactory.extVarNum(1);
			cachedBdd = Misc.bddFactory.ithVar(bddVar);
			Misc.representativeOfBdd(cachedBdd, this);
		}
		return cachedBdd;
	}
	
	@Override
	public int hashCode() {
		return operand.hashCode() * 2;
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof XOperator))
			return false;
		else
			return ((XOperator)o).operand.equals(this.operand);
	}
	
	@Override
	public Formula negated() {
		return new XOperator(operand.negated());
	}

               	@Override
        public String toReversePolishString() {
                    return "X " + operand.toReversePolishString();
        }

	@Override
	public String toString() {
		if (strFormula == null) strFormula = "X " + operand.toString();
		return strFormula;
	}

	@Override
	public Tuple<Set<Formula>, Set<Formula>> recurrentFormulas(boolean accumulate) {
		return operand.recurrentFormulas(accumulate);
	}

    @Override
    public Formula evaluateCurrentAssertions(Valuation valuation) {
        return this;
	}
	
	@Override
	public Formula removeConstantCurrentAssertions() {
		return this;
	}
	
	@Override
	public Formula removeX() {
		return operand;
	}

	@Override
	public Formula unfold() {
		return this;
	}

	@Override
	public Formula unfoldNoG() {
		return this;
	}
	
	@Override
	public boolean isProgressFormula() {
		return operand.isProgressFormula();
	}
	
	@Override
	public boolean containsG() {
		return operand.containsG();
	}
	

	@Override
	public boolean isVeryDifferentFrom(Formula f) {
		return ! f.hasSubformula(operand);
	}

        @Override
	public boolean hasSubformula(Formula f) {
            return this.equals(f) || operand.hasSubformula(f);
	}

        @Override
        public Formula removeXsFromCurrentBooleanAtoms() {
            if (operand.isProgressFormula()) return this;
            else return operand.removeXsFromCurrentBooleanAtoms();
        }

    @Override
	public Set<Formula> argumentsAlways() {
            return operand.argumentsAlways();
	}

        @Override
    public Set<Formula> argumentsFinsideG(boolean acc) {
        return operand.argumentsFinsideG(acc);
    }
        
    public boolean untilOcurrs() {
    	return operand.untilOcurrs();
    }
    
    public Formula toNNF() {
    	return new XOperator(operand.toNNF());
    }
}
