package tests;

import static org.junit.Assert.*;

import java.io.StringReader;
import java.util.*;

import org.junit.Test;

import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import rabinizer.exec.Valuation;
import rabinizer.formulas.Formula;
import rabinizer.parser.LTLParser;

@RunWith(value = Parameterized.class)
public class AssertValuation {

	private String input;
	private Valuation valuation;
	private String expected;	
	
	public AssertValuation(String input, String expected) {
		this.input = input;
		boolean[] values = {true, false};
		this.valuation = new Valuation(values);
		this.expected = expected;
	}

    @Parameters
	public static Collection<Object[]> data() {
        Object[][] data = new Object[][] { 
            { "false | a & G (a & F b)", "G (a & F b)" },
            { "F a | X b", "F a | X b" },
            { "G a U X G b", "G a U X G b" },
                                
            { "a & X b", "X b" },
            { "(a & X b) | (b & X X a)", "X b" },
            { "X a & G b", "X a & G b" },
            { "a & (b | X c)", "X c" },

            { "! two", "false" },
        };
        return Arrays.asList(data);
    }

	@Test
	public void pushTest() {
		System.out.println("\n"+this.getClass().getCanonicalName());
		
		LTLParser parser = new LTLParser(new StringReader(input));
		Formula inf = null;
		try { inf = parser.parse(); }
		catch (Exception e) { assertTrue(false); }

		parser.ReInit(new StringReader(expected));
		Formula exf = null;
		try { exf = parser.parsePreviousAtoms(); }
		catch (Exception e) { assertTrue(false); }
				
		System.out.println("input   : "+input);
		System.out.println("actual  : "+inf.assertValuation(valuation));
		System.out.println("expected: "+expected);
		assertTrue(exf.equals(inf.assertValuation(valuation)));
	}
}
